@@ -2,6 +2,8 @@
  *
  * Copyright (C) 2008-2010 Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -23,27 +25,28 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "giotypes.h"
-#include "gioenumtypes.h"
-#include "gsocket.h"
 #include "gdbusauthobserver.h"
-#include "gdbusprivate.h"
-#include "gdbusmessage.h"
 #include "gdbusconnection.h"
-#include "gdbusproxy.h"
+#include "gdbusdaemon.h"
 #include "gdbuserror.h"
 #include "gdbusintrospection.h"
-#include "gdbusdaemon.h"
-#include "giomodule-priv.h"
-#include "gtask.h"
+#include "gdbusmessage.h"
+#include "gdbusprivate.h"
+#include "gdbusproxy.h"
 #include "ginputstream.h"
-#include "gmemoryinputstream.h"
+#include "gioenumtypes.h"
+#include "giomodule-priv.h"
 #include "giostream.h"
+#include "giotypes.h"
+#include "glib-private.h"
 #include "glib/gstdio.h"
+#include "gmemoryinputstream.h"
+#include "gsocket.h"
 #include "gsocketaddress.h"
-#include "gsocketcontrolmessage.h"
 #include "gsocketconnection.h"
+#include "gsocketcontrolmessage.h"
 #include "gsocketoutputstream.h"
+#include "gtask.h"
 
 #ifdef G_OS_UNIX
 #include "gunixfdmessage.h"
@@ -55,6 +58,7 @@
 #include <windows.h>
 #include <io.h>
 #include <conio.h>
+#include "gwin32sid.h"
 #endif
 
 #include "glibintl.h"
@@ -183,6 +187,7 @@ _g_socket_read_with_control_messages (GSocket                 *socket,
 
   task = g_task_new (socket, cancellable, callback, user_data);
   g_task_set_source_tag (task, _g_socket_read_with_control_messages);
+  g_task_set_name (task, "[gio] D-Bus read");
   g_task_set_task_data (task, data, (GDestroyNotify) read_with_control_data_free);
 
   if (g_socket_condition_check (socket, G_IO_IN))
@@ -264,7 +269,7 @@ ensure_required_types (void)
 
 typedef struct
 {
-  volatile gint refcount;
+  gint refcount;  /* (atomic) */
   GThread *thread;
   GMainContext *context;
   GMainLoop *loop;
@@ -289,10 +294,9 @@ gdbus_shared_thread_func (gpointer user_data)
 static SharedThreadData *
 _g_dbus_shared_thread_ref (void)
 {
-  static gsize shared_thread_data = 0;
-  SharedThreadData *ret;
+  static SharedThreadData *shared_thread_data = 0;
 
-  if (g_once_init_enter (&shared_thread_data))
+  if (g_once_init_enter_pointer (&shared_thread_data))
     {
       SharedThreadData *data;
 
@@ -305,12 +309,11 @@ _g_dbus_shared_thread_ref (void)
                                    gdbus_shared_thread_func,
                                    data);
       /* We can cast between gsize and gpointer safely */
-      g_once_init_leave (&shared_thread_data, (gsize) data);
+      g_once_init_leave_pointer (&shared_thread_data, data);
     }
 
-  ret = (SharedThreadData*) shared_thread_data;
-  g_atomic_int_inc (&ret->refcount);
-  return ret;
+  g_atomic_int_inc (&shared_thread_data->refcount);
+  return shared_thread_data;
 }
 
 static void
@@ -340,12 +343,12 @@ typedef enum {
 
 struct GDBusWorker
 {
-  volatile gint                       ref_count;
+  gint                                ref_count;  /* (atomic) */
 
   SharedThreadData                   *shared_thread_data;
 
   /* really a boolean, but GLib 2.28 lacks atomic boolean ops */
-  volatile gint                       stopped;
+  gint                                stopped;  /* (atomic) */
 
   /* TODO: frozen (e.g. G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING) currently
    * only affects messages received from the other peer (since GDBusServer is the
@@ -553,7 +556,7 @@ _g_dbus_worker_unfreeze (GDBusWorker *worker)
                          unfreeze_in_idle_cb,
                          _g_dbus_worker_ref (worker),
                          (GDestroyNotify) _g_dbus_worker_unref);
-  g_source_set_name (idle_source, "[gio] unfreeze_in_idle_cb");
+  g_source_set_static_name (idle_source, "[gio] unfreeze_in_idle_cb");
   g_source_attach (idle_source, worker->shared_thread_data->context);
   g_source_unref (idle_source);
 }
@@ -884,21 +887,25 @@ _g_dbus_worker_do_initial_read (gpointer data)
 struct _MessageToWriteData
 {
   GDBusWorker  *worker;
-  GDBusMessage *message;
+  GDBusMessage *message;  /* (owned) */
   gchar        *blob;
   gsize         blob_size;
 
   gsize         total_written;
-  GTask        *task;
+  GTask        *task;  /* (owned) and (nullable) before writing starts and after g_task_return_*() is called */
 };
 
 static void
 message_to_write_data_free (MessageToWriteData *data)
 {
   _g_dbus_worker_unref (data->worker);
-  if (data->message)
-    g_object_unref (data->message);
+  g_clear_object (&data->message);
   g_free (data->blob);
+
+  /* The task must either not have been created, or have been created, returned
+   * and finalised by now. */
+  g_assert (data->task == NULL);
+
   g_slice_free (MessageToWriteData, data);
 }
 
@@ -910,21 +917,22 @@ static void write_message_continue_writing (MessageToWriteData *data);
  *
  * write-lock is not held on entry
  * output_pending is PENDING_WRITE on entry
+ * @user_data is (transfer full)
  */
 static void
 write_message_async_cb (GObject      *source_object,
                         GAsyncResult *res,
                         gpointer      user_data)
 {
-  MessageToWriteData *data = user_data;
-  GTask *task;
+  MessageToWriteData *data = g_steal_pointer (&user_data);
   gssize bytes_written;
   GError *error;
 
-  /* Note: we can't access data->task after calling g_task_return_* () because the
-   * callback can free @data and we're not completing in idle. So use a copy of the pointer.
-   */
-  task = data->task;
+  /* The ownership of @data is a bit odd in this function: it’s (transfer full)
+   * when the function is called, but the code paths which call g_task_return_*()
+   * on @data->task will indirectly cause it to be freed, because @data is
+   * always guaranteed to be the user_data in the #GTask. So that’s why it looks
+   * like @data is not always freed on every code path in this function. */
 
   error = NULL;
   bytes_written = g_output_stream_write_finish (G_OUTPUT_STREAM (source_object),
@@ -932,8 +940,9 @@ write_message_async_cb (GObject      *source_object,
                                                 &error);
   if (bytes_written == -1)
     {
+      GTask *task = g_steal_pointer (&data->task);
       g_task_return_error (task, error);
-      g_object_unref (task);
+      g_clear_object (&task);
       goto out;
     }
   g_assert (bytes_written > 0); /* zero is never returned */
@@ -944,12 +953,13 @@ write_message_async_cb (GObject      *source_object,
   g_assert (data->total_written <= data->blob_size);
   if (data->total_written == data->blob_size)
     {
+      GTask *task = g_steal_pointer (&data->task);
       g_task_return_boolean (task, TRUE);
-      g_object_unref (task);
+      g_clear_object (&task);
       goto out;
     }
 
-  write_message_continue_writing (data);
+  write_message_continue_writing (g_steal_pointer (&data));
 
  out:
   ;
@@ -966,9 +976,9 @@ on_socket_ready (GSocket      *socket,
                  GIOCondition  condition,
                  gpointer      user_data)
 {
-  MessageToWriteData *data = user_data;
-  write_message_continue_writing (data);
-  return FALSE; /* remove source */
+  MessageToWriteData *data = g_steal_pointer (&user_data);
+  write_message_continue_writing (g_steal_pointer (&data));
+  return G_SOURCE_REMOVE;
 }
 #endif
 
@@ -976,22 +986,21 @@ on_socket_ready (GSocket      *socket,
  *
  * write-lock is not held on entry
  * output_pending is PENDING_WRITE on entry
+ * @data is (transfer full)
  */
 static void
 write_message_continue_writing (MessageToWriteData *data)
 {
   GOutputStream *ostream;
 #ifdef G_OS_UNIX
-  GTask *task;
   GUnixFDList *fd_list;
 #endif
 
-#ifdef G_OS_UNIX
-  /* Note: we can't access data->task after calling g_task_return_* () because the
-   * callback can free @data and we're not completing in idle. So use a copy of the pointer.
-   */
-  task = data->task;
-#endif
+  /* The ownership of @data is a bit odd in this function: it’s (transfer full)
+   * when the function is called, but the code paths which call g_task_return_*()
+   * on @data->task will indirectly cause it to be freed, because @data is
+   * always guaranteed to be the user_data in the #GTask. So that’s why it looks
+   * like @data is not always freed on every code path in this function. */
 
   ostream = g_io_stream_get_output_stream (data->worker->stream);
 #ifdef G_OS_UNIX
@@ -1020,11 +1029,14 @@ write_message_continue_writing (MessageToWriteData *data)
         {
           if (!(data->worker->capabilities & G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING))
             {
-              g_task_return_new_error (task,
-                                       G_IO_ERROR,
-                                       G_IO_ERROR_FAILED,
-                                       "Tried sending a file descriptor but remote peer does not support this capability");
-              g_object_unref (task);
+              GTask *task = g_steal_pointer (&data->task);
+              g_task_return_new_error_literal (task,
+                                               G_IO_ERROR,
+                                               G_IO_ERROR_FAILED,
+                                               "Tried sending a file descriptor "
+                                               "but remote peer does not support "
+                                               "this capability");
+              g_clear_object (&task);
               goto out;
             }
           control_message = g_unix_fd_message_new_with_fd_list (fd_list);
@@ -1054,16 +1066,20 @@ write_message_continue_writing (MessageToWriteData *data)
                                                data->worker->cancellable);
               g_source_set_callback (source,
                                      (GSourceFunc) on_socket_ready,
-                                     data,
+                                     g_steal_pointer (&data),
                                      NULL); /* GDestroyNotify */
               g_source_attach (source, g_main_context_get_thread_default ());
               g_source_unref (source);
               g_error_free (error);
               goto out;
             }
-          g_task_return_error (task, error);
-          g_object_unref (task);
-          goto out;
+          else
+            {
+              GTask *task = g_steal_pointer (&data->task);
+              g_task_return_error (task, error);
+              g_clear_object (&task);
+              goto out;
+            }
         }
       g_assert (bytes_written > 0); /* zero is never returned */
 
@@ -1073,25 +1089,30 @@ write_message_continue_writing (MessageToWriteData *data)
       g_assert (data->total_written <= data->blob_size);
       if (data->total_written == data->blob_size)
         {
+          GTask *task = g_steal_pointer (&data->task);
           g_task_return_boolean (task, TRUE);
-          g_object_unref (task);
+          g_clear_object (&task);
           goto out;
         }
 
-      write_message_continue_writing (data);
+      write_message_continue_writing (g_steal_pointer (&data));
     }
 #endif
   else
     {
 #ifdef G_OS_UNIX
-      if (fd_list != NULL)
+      if (data->total_written == 0 && fd_list != NULL)
         {
+          /* We were trying to write byte 0 of the message, which needs
+           * the fd list to be attached to it, but this connection doesn't
+           * support doing that. */
+          GTask *task = g_steal_pointer (&data->task);
           g_task_return_new_error (task,
                                    G_IO_ERROR,
                                    G_IO_ERROR_FAILED,
                                    "Tried sending a file descriptor on unsupported stream of type %s",
                                    g_type_name (G_TYPE_FROM_INSTANCE (ostream)));
-          g_object_unref (task);
+          g_clear_object (&task);
           goto out;
         }
 #endif
@@ -1102,7 +1123,7 @@ write_message_continue_writing (MessageToWriteData *data)
                                    G_PRIORITY_DEFAULT,
                                    data->worker->cancellable,
                                    write_message_async_cb,
-                                   data);
+                                   data);  /* steal @data */
     }
 #ifdef G_OS_UNIX
  out:
@@ -1123,8 +1144,9 @@ write_message_async (GDBusWorker         *worker,
 {
   data->task = g_task_new (NULL, NULL, callback, user_data);
   g_task_set_source_tag (data->task, write_message_async);
+  g_task_set_name (data->task, "[gio] D-Bus write message");
   data->total_written = 0;
-  write_message_continue_writing (data);
+  write_message_continue_writing (g_steal_pointer (&data));
 }
 
 /* called in private thread shared by all GDBusConnection instances (with write-lock held) */
@@ -1313,6 +1335,7 @@ prepare_flush_unlocked (GDBusWorker *worker)
  *
  * write-lock is not held on entry
  * output_pending is PENDING_WRITE on entry
+ * @user_data is (transfer full)
  */
 static void
 write_message_cb (GObject       *source_object,
@@ -1504,7 +1527,7 @@ continue_writing (GDBusWorker *worker)
         }
       else
         {
-          /* filters altered the message -> reencode */
+          /* filters altered the message -> re-encode */
           error = NULL;
           new_blob = g_dbus_message_to_blob (data->message,
                                              &new_blob_size,
@@ -1531,7 +1554,7 @@ continue_writing (GDBusWorker *worker)
       write_message_async (worker,
                            data,
                            write_message_cb,
-                           data);
+                           data);  /* takes ownership of @data as user_data */
     }
 }
 
@@ -1596,7 +1619,7 @@ schedule_writing_unlocked (GDBusWorker        *worker,
                              continue_writing_in_idle_cb,
                              _g_dbus_worker_ref (worker),
                              (GDestroyNotify) _g_dbus_worker_unref);
-      g_source_set_name (idle_source, "[gio] continue_writing_in_idle_cb");
+      g_source_set_static_name (idle_source, "[gio] continue_writing_in_idle_cb");
       g_source_attach (idle_source, worker->shared_thread_data->context);
       g_source_unref (idle_source);
     }
@@ -1691,7 +1714,7 @@ _g_dbus_worker_new (GIOStream                              *stream,
                          _g_dbus_worker_do_initial_read,
                          _g_dbus_worker_ref (worker),
                          (GDestroyNotify) _g_dbus_worker_unref);
-  g_source_set_name (idle_source, "[gio] _g_dbus_worker_do_initial_read");
+  g_source_set_static_name (idle_source, "[gio] _g_dbus_worker_do_initial_read");
   g_source_attach (idle_source, worker->shared_thread_data->context);
   g_source_unref (idle_source);
 
@@ -1936,15 +1959,14 @@ _g_dbus_debug_print_unlock (void)
 void
 _g_dbus_initialize (void)
 {
-  static volatile gsize initialized = 0;
+  static gsize initialized = 0;
 
   if (g_once_init_enter (&initialized))
     {
-      volatile GQuark g_dbus_error_domain;
       const gchar *debug;
 
-      g_dbus_error_domain = G_DBUS_ERROR;
-      (g_dbus_error_domain); /* To avoid -Wunused-but-set-variable */
+      /* Ensure the domain is registered. */
+      g_dbus_error_quark ();
 
       debug = g_getenv ("G_DBUS_DEBUG");
       if (debug != NULL)
@@ -2006,73 +2028,10 @@ _g_dbus_compute_complete_signature (GDBusArgInfo **args)
 
 #ifdef G_OS_WIN32
 
-extern BOOL WINAPI ConvertSidToStringSidA (PSID Sid, LPSTR *StringSid);
-
-gchar *
-_g_dbus_win32_get_user_sid (void)
-{
-  HANDLE h;
-  TOKEN_USER *user;
-  DWORD token_information_len;
-  PSID psid;
-  gchar *sid;
-  gchar *ret;
-
-  ret = NULL;
-  user = NULL;
-  h = INVALID_HANDLE_VALUE;
-
-  if (!OpenProcessToken (GetCurrentProcess (), TOKEN_QUERY, &h))
-    {
-      g_warning ("OpenProcessToken failed with error code %d", (gint) GetLastError ());
-      goto out;
-    }
-
-  /* Get length of buffer */
-  token_information_len = 0;
-  if (!GetTokenInformation (h, TokenUser, NULL, 0, &token_information_len))
-    {
-      if (GetLastError () != ERROR_INSUFFICIENT_BUFFER)
-        {
-          g_warning ("GetTokenInformation() failed with error code %d", (gint) GetLastError ());
-          goto out;
-        }
-    }
-  user = g_malloc (token_information_len);
-  if (!GetTokenInformation (h, TokenUser, user, token_information_len, &token_information_len))
-    {
-      g_warning ("GetTokenInformation() failed with error code %d", (gint) GetLastError ());
-      goto out;
-    }
-
-  psid = user->User.Sid;
-  if (!IsValidSid (psid))
-    {
-      g_warning ("Invalid SID");
-      goto out;
-    }
-
-  if (!ConvertSidToStringSidA (psid, &sid))
-    {
-      g_warning ("Invalid SID");
-      goto out;
-    }
-
-  ret = g_strdup (sid);
-  LocalFree (sid);
-
-out:
-  g_free (user);
-  if (h != INVALID_HANDLE_VALUE)
-    CloseHandle (h);
-  return ret;
-}
-
-
-#define DBUS_DAEMON_ADDRESS_INFO "DBusDaemonAddressInfo"
-#define DBUS_DAEMON_MUTEX "DBusDaemonMutex"
-#define UNIQUE_DBUS_INIT_MUTEX "UniqueDBusInitMutex"
-#define DBUS_AUTOLAUNCH_MUTEX "DBusAutolaunchMutex"
+#define DBUS_DAEMON_ADDRESS_INFO L"DBusDaemonAddressInfo"
+#define DBUS_DAEMON_MUTEX L"DBusDaemonMutex"
+#define UNIQUE_DBUS_INIT_MUTEX L"UniqueDBusInitMutex"
+#define DBUS_AUTOLAUNCH_MUTEX L"DBusAutolaunchMutex"
 
 static void
 release_mutex (HANDLE mutex)
@@ -2082,12 +2041,12 @@ release_mutex (HANDLE mutex)
 }
 
 static HANDLE
-acquire_mutex (const char *mutexname)
+acquire_mutex (const wchar_t *mutexname)
 {
   HANDLE mutex;
   DWORD res;
 
-  mutex = CreateMutexA (NULL, FALSE, mutexname);
+  mutex = CreateMutex (NULL, FALSE, mutexname);
   if (!mutex)
     return 0;
 
@@ -2106,12 +2065,12 @@ acquire_mutex (const char *mutexname)
 }
 
 static gboolean
-is_mutex_owned (const char *mutexname)
+is_mutex_owned (const wchar_t *mutexname)
 {
   HANDLE mutex;
   gboolean res = FALSE;
 
-  mutex = CreateMutexA (NULL, FALSE, mutexname);
+  mutex = CreateMutex (NULL, FALSE, mutexname);
   if (WaitForSingleObject (mutex, 10) == WAIT_TIMEOUT)
     res = TRUE;
   else
@@ -2122,7 +2081,7 @@ is_mutex_owned (const char *mutexname)
 }
 
 static char *
-read_shm (const char *shm_name)
+read_shm (const wchar_t *shm_name)
 {
   HANDLE shared_mem;
   char *shared_data;
@@ -2133,7 +2092,7 @@ read_shm (const char *shm_name)
 
   for (i = 0; i < 20; i++)
     {
-      shared_mem = OpenFileMappingA (FILE_MAP_READ, FALSE, shm_name);
+      shared_mem = OpenFileMapping (FILE_MAP_READ, FALSE, shm_name);
       if (shared_mem != 0)
 	break;
       Sleep (100);
@@ -2163,13 +2122,13 @@ read_shm (const char *shm_name)
 }
 
 static HANDLE
-set_shm (const char *shm_name, const char *value)
+set_shm (const wchar_t *shm_name, const char *value)
 {
   HANDLE shared_mem;
   char *shared_data;
 
-  shared_mem = CreateFileMappingA (INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
-				   0, strlen (value) + 1, shm_name);
+  shared_mem = CreateFileMapping (INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
+                                  0, strlen (value) + 1, shm_name);
   if (shared_mem == 0)
     return 0;
 
@@ -2195,7 +2154,7 @@ publish_session_bus (const char *address)
 
   init_mutex = acquire_mutex (UNIQUE_DBUS_INIT_MUTEX);
 
-  published_daemon_mutex = CreateMutexA (NULL, FALSE, DBUS_DAEMON_MUTEX);
+  published_daemon_mutex = CreateMutex (NULL, FALSE, DBUS_DAEMON_MUTEX);
   if (WaitForSingleObject (published_daemon_mutex, 10 ) != WAIT_OBJECT_0)
     {
       release_mutex (init_mutex);
@@ -2236,7 +2195,7 @@ unpublish_session_bus (void)
 static void
 wait_console_window (void)
 {
-  FILE *console = fopen ("CONOUT$", "w");
+  FILE *console = g_fopen ("CONOUT$", "we");
 
   SetConsoleTitleW (L"gdbus-daemon output. Type any character to close this window.");
   fprintf (console, _("(Type any character to close this window)\n"));
@@ -2288,7 +2247,7 @@ turn_off_the_starting_cursor (void)
     }
 }
 
-__declspec(dllexport) void __stdcall
+void __stdcall
 g_win32_run_session_bus (void* hwnd, void* hinst, const char* cmdline, int cmdshow)
 {
   GDBusDaemon *daemon;
@@ -2333,6 +2292,26 @@ g_win32_run_session_bus (void* hwnd, void* hinst, const char* cmdline, int cmdsh
 
 static gboolean autolaunch_binary_absent = FALSE;
 
+static wchar_t *
+find_dbus_process_path (void)
+{
+  wchar_t *dbus_path;
+  gchar *exe_path = GLIB_PRIVATE_CALL (g_win32_find_helper_executable_path) ("gdbus.exe", _g_io_win32_get_module ());
+  dbus_path = g_utf8_to_utf16 (exe_path, -1, NULL, NULL, NULL);
+  g_free (exe_path);
+
+  if (dbus_path == NULL)
+    return NULL;
+
+  if (GetFileAttributesW (dbus_path) == INVALID_FILE_ATTRIBUTES)
+    {
+      g_free (dbus_path);
+      return NULL;
+    }
+
+  return dbus_path;
+}
+
 gchar *
 _g_dbus_win32_get_session_address_dbus_launch (GError **error)
 {
@@ -2350,63 +2329,63 @@ _g_dbus_win32_get_session_address_dbus_launch (GError **error)
 
   if (address == NULL && !autolaunch_binary_absent)
     {
-      wchar_t gio_path[MAX_PATH + 2] = { 0 };
-      int gio_path_len = GetModuleFileNameW (_g_io_win32_get_module (), gio_path, MAX_PATH + 1);
+      wchar_t *dbus_path;
+      size_t dbus_path_len;
+      PROCESS_INFORMATION pi = { 0 };
+      STARTUPINFOW si = { 0 };
+      BOOL res = FALSE;
+      wchar_t args[MAX_PATH * 2 + 100] = { 0 };
+      wchar_t working_dir[MAX_PATH + 2] = { 0 };
+      wchar_t *p;
 
-      /* The <= MAX_PATH check prevents truncated path usage */
-      if (gio_path_len > 0 && gio_path_len <= MAX_PATH)
-	{
-	  PROCESS_INFORMATION pi = { 0 };
-	  STARTUPINFOW si = { 0 };
-	  BOOL res = FALSE;
-	  wchar_t exe_path[MAX_PATH + 100] = { 0 };
-	  /* calculate index of first char of dll file name inside full path */
-	  int gio_name_index = gio_path_len;
-	  for (; gio_name_index > 0; --gio_name_index)
-	  {
-	    wchar_t prev_char = gio_path[gio_name_index - 1];
-	    if (prev_char == L'\\' || prev_char == L'/')
-	      break;
-	  }
-	  gio_path[gio_name_index] = L'\0';
-	  wcscpy (exe_path, gio_path);
-	  wcscat (exe_path, L"\\gdbus.exe");
+      dbus_path = find_dbus_process_path ();
+      if (dbus_path == NULL)
+        {
+          /* warning won't be raised another time
+           * since autolaunch_binary_absent would be already set.
+           */
+          autolaunch_binary_absent = TRUE;
+          g_warning ("win32 session dbus binary not found");
+          goto out;
+        }
 
-	  if (GetFileAttributesW (exe_path) == INVALID_FILE_ATTRIBUTES)
-	    {
-	      /* warning won't be raised another time
-	       * since autolaunch_binary_absent would be already set.
-	       */
-	      autolaunch_binary_absent = TRUE;
-	      g_warning ("win32 session dbus binary not found: %S", exe_path );
-	    }
-	  else
-	    {
-	      wchar_t args[MAX_PATH*2 + 100] = { 0 };
-	      wcscpy (args, L"\"");
-	      wcscat (args, exe_path);
-	      wcscat (args, L"\" ");
+      dbus_path_len = wcslen (dbus_path);
+      if (dbus_path_len + 1 > sizeof (working_dir))
+        {
+          g_warning ("Path to win32 session dbus binary is too long");
+          goto out;
+        }
+
+      wcscpy (working_dir, dbus_path);
+      p = wcsrchr (working_dir, L'\\');
+      if (p != NULL)
+        *p = L'\0';
+
+      wcscpy (args, L"\"");
+      wcscat (args, dbus_path);
+      wcscat (args, L"\" ");
 #define _L_PREFIX_FOR_EXPANDED(arg) L##arg
 #define _L_PREFIX(arg) _L_PREFIX_FOR_EXPANDED (arg)
-	      wcscat (args, _L_PREFIX (_GDBUS_ARG_WIN32_RUN_SESSION_BUS));
+      wcscat (args, _L_PREFIX (_GDBUS_ARG_WIN32_RUN_SESSION_BUS));
 #undef _L_PREFIX
 #undef _L_PREFIX_FOR_EXPANDED
 
-	      res = CreateProcessW (exe_path, args,
-				    0, 0, FALSE,
-				    NORMAL_PRIORITY_CLASS | CREATE_NO_WINDOW | DETACHED_PROCESS,
-				    0, gio_path,
-				    &si, &pi);
-	    }
-	  if (res)
-	    {
-	      address = read_shm (DBUS_DAEMON_ADDRESS_INFO);
-	      if (address == NULL)
-		g_warning ("%S dbus binary failed to launch bus, maybe incompatible version", exe_path );
-	    }
-	}
-    }
+      res = CreateProcessW (dbus_path, args,
+                            0, 0, FALSE,
+                            NORMAL_PRIORITY_CLASS | CREATE_NO_WINDOW | DETACHED_PROCESS,
+                            0, working_dir,
+                            &si, &pi);
 
+      if (res)
+        {
+          address = read_shm (DBUS_DAEMON_ADDRESS_INFO);
+          if (address == NULL)
+            g_warning ("%S dbus binary failed to launch bus, maybe incompatible version", dbus_path);
+        }
+
+      g_free (dbus_path);
+    }
+out:
   release_mutex (autolaunch_mutex);
 
   if (address == NULL)
@@ -2426,11 +2405,11 @@ gchar *
 _g_dbus_get_machine_id (GError **error)
 {
 #ifdef G_OS_WIN32
-  HW_PROFILE_INFOA info;
-  char *src, *dest, *res;
+  HW_PROFILE_INFO info;
+  char *guid, *src, *dest, *res;
   int i;
 
-  if (!GetCurrentHwProfileA (&info))
+  if (!GetCurrentHwProfile (&info))
     {
       char *message = g_win32_error_message (GetLastError ());
       g_set_error (error,
@@ -2441,8 +2420,11 @@ _g_dbus_get_machine_id (GError **error)
       return NULL;
     }
 
-  /* Form: {12340001-4980-1920-6788-123456789012} */
-  src = &info.szHwProfileGuid[0];
+  if (!(guid = g_utf16_to_utf8 (info.szHwProfileGuid, -1, NULL, NULL, NULL)))
+    return NULL;
+
+  /* Guid is of the form: {12340001-4980-1920-6788-123456789012} */
+  src = guid;
 
   res = g_malloc (32+1);
   dest = res;
@@ -2464,33 +2446,67 @@ _g_dbus_get_machine_id (GError **error)
     *dest++ = *src++;
   *dest = 0;
 
+  g_free (guid);
+
   return res;
 #else
-  gchar *ret;
-  GError *first_error;
-  /* TODO: use PACKAGE_LOCALSTATEDIR ? */
-  ret = NULL;
-  first_error = NULL;
-  if (!g_file_get_contents ("/var/lib/dbus/machine-id",
+  gchar *ret = NULL;
+  GError *first_error = NULL;
+  gsize i;
+  gboolean non_zero = FALSE;
+
+  /* Copy what dbus.git does: allow the /var/lib path to be configurable at
+   * build time, but hard-code the system-wide machine ID path in /etc. */
+  const gchar *var_lib_path = LOCALSTATEDIR "/lib/dbus/machine-id";
+  const gchar *etc_path = "/etc/machine-id";
+
+  if (!g_file_get_contents (var_lib_path,
                             &ret,
                             NULL,
                             &first_error) &&
-      !g_file_get_contents ("/etc/machine-id",
+      !g_file_get_contents (etc_path,
                             &ret,
                             NULL,
                             NULL))
     {
-      g_propagate_prefixed_error (error, first_error,
-                                  _("Unable to load /var/lib/dbus/machine-id or /etc/machine-id: "));
+      g_propagate_prefixed_error (error, g_steal_pointer (&first_error),
+                                  /* Translators: Both placeholders are file paths */
+                                  _("Unable to load %s or %s: "),
+                                  var_lib_path, etc_path);
+      return NULL;
     }
-  else
+
+  /* ignore the error from the first try, if any */
+  g_clear_error (&first_error);
+
+  /* Validate the machine ID. From `man 5 machine-id`:
+   * > The machine ID is a single newline-terminated, hexadecimal, 32-character,
+   * > lowercase ID. When decoded from hexadecimal, this corresponds to a
+   * > 16-byte/128-bit value. This ID may not be all zeros.
+   */
+  for (i = 0; ret[i] != '\0' && ret[i] != '\n'; i++)
     {
-      /* ignore the error from the first try, if any */
-      g_clear_error (&first_error);
-      /* TODO: validate value */
-      g_strstrip (ret);
+      /* Break early if it’s invalid. */
+      if (!g_ascii_isxdigit (ret[i]) || g_ascii_isupper (ret[i]))
+        break;
+
+      if (ret[i] != '0')
+        non_zero = TRUE;
     }
-  return ret;
+
+  if (i != 32 || ret[i] != '\n' || ret[i + 1] != '\0' || !non_zero)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Invalid machine ID in %s or %s",
+                   var_lib_path, etc_path);
+      g_free (ret);
+      return NULL;
+    }
+
+  /* Strip trailing newline. */
+  ret[32] = '\0';
+
+  return g_steal_pointer (&ret);
 #endif
 }
 
