@@ -86,7 +86,7 @@ static inline void
 g_string_maybe_expand (GString *string,
                        gsize    len)
 {
-  if (G_UNLIKELY (string->len + len >= string->allocated_len))
+  if (G_UNLIKELY (len >= string->allocated_len - string->len))
     g_string_expand (string, len);
 }
 
@@ -154,7 +154,7 @@ g_string_new (const gchar *init)
  * Creates a new #GString, initialized with the given string.
  *
  * After this call, @init belongs to the #GString and may no longer be
- * modified by the caller. The memory of @data has to be dynamically
+ * modified by the caller. The memory of @init has to be dynamically
  * allocated and will eventually be freed with g_free().
  *
  * Returns: (transfer full): the new #GString
@@ -214,6 +214,38 @@ g_string_new_len (const gchar *init,
     }
 }
 
+/**
+ * g_string_copy:
+ * @string: a string
+ *
+ * Copies the [struct@GLib.String] instance and its contents.
+ *
+ * This will preserve the allocation length of the [struct@GLib.String] in the
+ * copy.
+ *
+ * Returns: (transfer full): a copy of @string
+ * Since: 2.86
+ */
+GString *
+g_string_copy (GString *string)
+{
+  GString *copy = NULL;
+
+  g_return_val_if_fail (string != NULL, NULL);
+
+  copy = g_slice_new (GString);
+  copy->allocated_len = string->allocated_len;
+  copy->len = string->len;
+
+  /* We can’t just strdup(string->str) here because it may contain embedded nuls. */
+  copy->str = g_malloc (string->allocated_len);
+  if (string->str != NULL && string->len > 0)
+    memcpy (copy->str, string->str, string->len);
+  copy->str[copy->len] = '\0';
+
+  return g_steal_pointer (&copy);
+}
+
 /**
  * g_string_free:
  * @string: (transfer full): a #GString
@@ -480,8 +512,9 @@ g_string_insert_len (GString     *string,
     return string;
 
   if (len < 0)
-    len = strlen (val);
-  len_unsigned = len;
+    len_unsigned = strlen (val);
+  else
+    len_unsigned = len;
 
   if (pos < 0)
     pos_unsigned = string->len;
@@ -778,10 +811,12 @@ g_string_insert_c (GString *string,
   g_string_maybe_expand (string, 1);
 
   if (pos < 0)
-    pos = string->len;
+    pos_unsigned = string->len;
   else
-    g_return_val_if_fail ((gsize) pos <= string->len, string);
-  pos_unsigned = pos;
+    {
+      pos_unsigned = pos;
+      g_return_val_if_fail (pos_unsigned <= string->len, string);
+    }
 
   /* If not just an append, move the old stuff */
   if (pos_unsigned < string->len)
@@ -814,6 +849,7 @@ g_string_insert_unichar (GString  *string,
                          gssize    pos,
                          gunichar  wc)
 {
+  gsize pos_unsigned;
   gint charlen, first, i;
   gchar *dest;
 
@@ -855,15 +891,18 @@ g_string_insert_unichar (GString  *string,
   g_string_maybe_expand (string, charlen);
 
   if (pos < 0)
-    pos = string->len;
+    pos_unsigned = string->len;
   else
-    g_return_val_if_fail ((gsize) pos <= string->len, string);
+    {
+      pos_unsigned = pos;
+      g_return_val_if_fail (pos_unsigned <= string->len, string);
+    }
 
   /* If not just an append, move the old stuff */
-  if ((gsize) pos < string->len)
-    memmove (string->str + pos + charlen, string->str + pos, string->len - pos);
+  if (pos_unsigned < string->len)
+    memmove (string->str + pos_unsigned + charlen, string->str + pos_unsigned, string->len - pos_unsigned);
 
-  dest = string->str + pos;
+  dest = string->str + pos_unsigned;
   /* Code copied from g_unichar_to_utf() */
   for (i = charlen - 1; i > 0; --i)
     {
@@ -921,6 +960,7 @@ g_string_overwrite_len (GString     *string,
                         const gchar *val,
                         gssize       len)
 {
+  gsize len_unsigned;
   gsize end;
 
   g_return_val_if_fail (string != NULL, NULL);
@@ -932,14 +972,16 @@ g_string_overwrite_len (GString     *string,
   g_return_val_if_fail (pos <= string->len, string);
 
   if (len < 0)
-    len = strlen (val);
+    len_unsigned = strlen (val);
+  else
+    len_unsigned = len;
 
-  end = pos + len;
+  end = pos + len_unsigned;
 
   if (end > string->len)
     g_string_maybe_expand (string, end - string->len);
 
-  memcpy (string->str + pos, val, len);
+  memcpy (string->str + pos, val, len_unsigned);
 
   if (end > string->len)
     {
@@ -1013,7 +1055,12 @@ g_string_erase (GString *string,
  * (beginning of string, end of string and between characters). This did
  * not work correctly in earlier versions.
  *
- * Returns: the number of find and replace operations performed.
+ * If @limit is zero and more than `G_MAXUINT` instances of @find are in
+ * the input string, they will all be replaced, but the return value will
+ * be capped at `G_MAXUINT`.
+ *
+ * Returns: the number of find and replace operations performed,
+ *   up to `G_MAXUINT`
  *
  * Since: 2.68
  */
@@ -1045,36 +1092,38 @@ g_string_replace (GString     *string,
    * handle the case of an empty @find string separately. */
   if (G_UNLIKELY (f_len == 0))
     {
-      if (limit == 0 || limit > string->len)
+      size_t r_limit = limit;
+
+      if (r_limit == 0 || r_limit > string->len)
         {
           if (string->len > G_MAXSIZE - 1)
             g_error ("inserting in every position in string would overflow");
 
-          limit = string->len + 1;
+          r_limit = string->len + 1;
         }
 
       if (r_len > 0 &&
-          (limit > G_MAXSIZE / r_len ||
-           limit * r_len > G_MAXSIZE - string->len))
+          (r_limit > G_MAXSIZE / r_len ||
+           r_limit * r_len > G_MAXSIZE - string->len))
         g_error ("inserting in every position in string would overflow");
 
-      new_len = string->len + limit * r_len;
+      new_len = string->len + r_limit * r_len;
       new_string = g_string_sized_new (new_len);
-      for (size_t i = 0; i < limit; i++)
+      for (size_t i = 0; i < r_limit; i++)
         {
           g_string_append_len (new_string, replace, r_len);
           if (i < string->len)
             g_string_append_c (new_string, string->str[i]);
         }
-      if (limit < string->len)
-        g_string_append_len (new_string, string->str + limit, string->len - limit);
+      if (r_limit < string->len)
+        g_string_append_len (new_string, string->str + r_limit, string->len - r_limit);
 
       g_free (string->str);
       string->allocated_len = new_string->allocated_len;
       string->len = new_string->len;
       string->str = g_string_free_and_steal (g_steal_pointer (&new_string));
 
-      return limit;
+      return r_limit > G_MAXUINT ? G_MAXUINT : (guint) r_limit;
     }
 
   /* Potentially do two passes: the first to calculate the length of the new string,
@@ -1095,7 +1144,8 @@ g_string_replace (GString     *string,
       n = 0;
       while ((next = strstr (cur, find)) != NULL)
         {
-          n++;
+          if (n < G_MAXUINT)
+            n++;
 
           if (r_len <= f_len)
             {
@@ -1318,7 +1368,7 @@ g_string_append_vprintf (GString     *string,
   if (len >= 0)
     {
       g_string_maybe_expand (string, len);
-      memcpy (string->str + string->len, buf, len + 1);
+      memcpy (string->str + string->len, buf, (size_t) len + 1);
       string->len += len;
       g_free (buf);
     }
