@@ -464,7 +464,8 @@ g_system_thread_new (GThreadFunc proxy,
   base_thread->thread.joinable = TRUE;
   base_thread->thread.func = func;
   base_thread->thread.data = data;
-  base_thread->name = g_strdup (name);
+  if (name)
+    g_strlcpy (base_thread->name, name, 16);
 
   thread->handle = (HANDLE) _beginthreadex (NULL, stack_size, g_thread_win32_proxy, thread,
                                             CREATE_SUSPENDED, &ignore);
@@ -488,16 +489,19 @@ g_system_thread_new (GThreadFunc proxy,
     thread_prio = GetThreadPriority (current_thread);
   }
 
-  if (thread_prio == THREAD_PRIORITY_ERROR_RETURN)
+  if (thread_prio != THREAD_PRIORITY_ERROR_RETURN)
     {
-      message = "Error getting current thread priority";
-      goto error;
-    }
-
-  if (SetThreadPriority (thread->handle, thread_prio) == 0)
-    {
-      message = "Error setting new thread priority";
-      goto error;
+      /* GetThreadPriority() may return non-predefined priority level
+       * which is not acceptable by SetThreadPriority().
+       * Ignores any errors occurred here since it's not a critical
+       */
+      if (!SetThreadPriority (thread->handle, thread_prio))
+        {
+          gchar *win_error = g_win32_error_message (GetLastError ());
+          g_debug ("SetThreadPriority with priority %d failed: %s",
+                   thread_prio, win_error);
+          g_free (win_error);
+        }
     }
 
   if (ResumeThread (thread->handle) == (DWORD) -1)
@@ -649,6 +653,15 @@ g_system_thread_set_name (const gchar *name)
     SetThreadName ((DWORD) -1, name);
 }
 
+void
+g_system_thread_get_name (char  *buffer,
+                          gsize  length)
+{
+  /* FIXME: Not implemented yet */
+  g_assert (length >= 1);
+  buffer[0] = '\0';
+}
+
 /* {{{1 Epilogue */
 
 void
